@@ -214,6 +214,14 @@ class ListField(Field):
 
         return instances
 
+    def __getitem__(self, key):
+        return getattr(self, key)
+
+
+class LinksField(CompositeField):
+    """Reference to linked resources."""
+    oem_vendors = Field('Oem', adapter=list)
+
 
 class DictionaryField(Field):
     """Base class for fields consisting of dictionary of several sub-fields."""
@@ -247,6 +255,9 @@ class DictionaryField(Field):
 
         return instances
 
+    def __getitem__(self, key):
+        return getattr(self, key)
+
 
 class MappedField(Field):
     """Field taking real value from a mapping."""
@@ -319,7 +330,7 @@ class MappedListField(Field):
 class MessageListField(ListField):
     """List of messages with details of settings update status"""
 
-    message_id = Field('MessageId', required=True)
+    message_id = Field('MessageId')
     """The key for this message which can be used
     to look up the message in a message registry
     """
@@ -477,9 +488,11 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
     redfish_version = None
     """The Redfish version"""
 
-    oem_vendors = Field('Oem', adapter=list)
+    _oem_vendors = Field('Oem', adapter=list)
     """The list of OEM extension names for this resource."""
 
+    links = LinksField('Links')
+
     def __init__(self,
                  connector,
                  path='',
@@ -514,17 +527,74 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
 
         self.refresh(json_doc=json_doc)
 
+    def _get_value(self, val):
+        """Iterate through the input to get values for all attributes
+
+        :param val: Either a value or a resource
+        :returns: Attribute value, which may be a dictionary
+        """
+        if isinstance(val, dict):
+            subfields = {}
+            for key, s_val in val.items():
+                subfields[key] = self._get_value(s_val)
+            return subfields
+
+        elif isinstance(val, list):
+            return [self._get_value(val[i]) for i in range(len(val))]
+
+        elif (isinstance(val, DictionaryField)
+              or isinstance(val, CompositeField)
+              or isinstance(val, ListField)):
+            subfields = {}
+            for attr, field in val._subfields.items():
+                subfields[attr] = self._get_value(val.__getitem__(attr))
+            return subfields
+
+        return val
+
     def _parse_attributes(self, json_doc):
         """Parse the attributes of a resource.
 
         Parsed JSON fields are set to `self` as declared in the class.
 
         :param json_doc: parsed JSON document in form of Python types
+        :returns: dictionary of attribute/values after parsing
         """
+        settings = {}
         for attr, field in _collect_fields(self):
             # Hide the Field object behind the real value
             setattr(self, attr, field._load(json_doc, self))
 
+            # Get the attribute/value pairs that have been parsed
+            settings[attr] = self._get_value(getattr(self, attr))
+
+        return settings
+
+    def _get_etag(self):
+        """Returns the ETag of the HTTP request if any was specified.
+
+        :returns ETag or None
+        """
+        return self._get_headers().get('ETag')
+
+    def _get_headers(self):
+        """Returns the HTTP headers of the request for the resource.
+
+        :returns: dict of HTTP headers
+        """
+        return self._reader.get_data()._headers
+
+    def _allow_patch(self):
+        """Returns if the resource supports the PATCH HTTP method.
+
+        If the resource supports the PATCH HTTP method for updates,
+        it will return it in the Allow HTTP header.
+        :returns: Boolean flag if PATCH is supported or not
+        """
+        allow_header = self._get_headers().get('Allow', '')
+        methods = set([h.strip().upper() for h in allow_header.split(',')])
+        return "PATCH" in methods
+
     def refresh(self, force=True, json_doc=None):
         """Refresh the resource
 
@@ -553,10 +623,10 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
         else:
             self._json = self._reader.get_data().json_doc
 
+        attributes = self._parse_attributes(self._json)
         LOG.debug('Received representation of %(type)s %(path)s: %(json)s',
                   {'type': self.__class__.__name__,
-                   'path': self._path, 'json': self._json})
-        self._parse_attributes(self._json)
+                   'path': self._path, 'json': attributes})
         self._do_refresh(force)
 
         # Mark it fresh
@@ -600,6 +670,12 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
         if force_refresh:
             self.refresh()
 
+    @property
+    def oem_vendors(self):
+        return list(
+            set((self._oem_vendors or []) + (self.links.oem_vendors or []))
+        )
+
     @property
     def json(self):
         return self._json
