@@ -14,8 +14,9 @@
 #    under the License.
 
 # This is referred from Redfish standard schema.
-# https://redfish.dmtf.org/schemas/ComputerSystem.v1_5_0.json
+# https://redfish.dmtf.org/schemas/v1/ComputerSystem.v1_10_0.json
 
+import collections
 import logging
 
 from sushy import exceptions
@@ -30,6 +31,7 @@ from sushy.resources.system import constants as sys_cons
 from sushy.resources.system import ethernet_interface
 from sushy.resources.system import mappings as sys_maps
 from sushy.resources.system import processor
+from sushy.resources.system import secure_boot
 from sushy.resources.system import simple_storage as sys_simple_storage
 from sushy.resources.system.storage import storage as sys_storage
 from sushy import utils
@@ -133,6 +135,12 @@ class System(base.ResourceBase):
     """Indicates if a given resource has a maintenance window assignment
     for applying settings or operations"""
 
+    _settings = settings.SettingsField()
+    """Settings Resource is used to represent the future intended state
+    of a Resource
+    Ref: http://redfish.dmtf.org/schemas/DSP0266_1.7.0.html#settings-resource
+    """
+
     _actions = ActionsField('Actions', required=True)
 
     def __init__(self, connector, identity, redfish_version=None,
@@ -207,41 +215,64 @@ class System(base.ResourceBase):
                     set(sys_maps.BOOT_SOURCE_TARGET_MAP).
                     intersection(self.boot.allowed_values)])
 
-    def set_system_boot_source(self, target,
-                               enabled=sys_cons.BOOT_SOURCE_ENABLED_ONCE,
-                               mode=None):
-        """Set the boot source.
+    def set_system_boot_options(self, target=None, enabled=None, mode=None):
+        """Set boot source and/or boot frequency and/or boot mode.
 
-        Set the boot source to use on next reboot of the System.
+        Set the boot source and/or boot frequency and/or boot mode to use
+        on next reboot of the System.
 
-        :param target: The target boot source.
+        :param target: The target boot source, optional.
         :param enabled: The frequency, whether to set it for the next
             reboot only (BOOT_SOURCE_ENABLED_ONCE) or persistent to all
             future reboots (BOOT_SOURCE_ENABLED_CONTINUOUS) or disabled
-            (BOOT_SOURCE_ENABLED_DISABLED).
-        :param mode: The boot mode, UEFI (BOOT_SOURCE_MODE_UEFI) or
-            BIOS (BOOT_SOURCE_MODE_BIOS).
+            (BOOT_SOURCE_ENABLED_DISABLED), optional.
+        :param mode: The boot mode (UEFI: BOOT_SOURCE_MODE_UEFI or
+            BIOS: BOOT_SOURCE_MODE_BIOS), optional.
         :raises: InvalidParameterValueError, if any information passed is
             invalid.
         """
-        valid_targets = self.get_allowed_system_boot_source_values()
-        if target not in valid_targets:
-            raise exceptions.InvalidParameterValueError(
-                parameter='target', value=target, valid_values=valid_targets)
+        data = collections.defaultdict(dict)
 
-        if enabled not in sys_maps.BOOT_SOURCE_ENABLED_MAP_REV:
-            raise exceptions.InvalidParameterValueError(
-                parameter='enabled', value=enabled,
-                valid_values=list(sys_maps.BOOT_SOURCE_ENABLED_MAP_REV))
+        if target is not None:
+            valid_targets = self.get_allowed_system_boot_source_values()
+            if target not in valid_targets:
+                raise exceptions.InvalidParameterValueError(
+                    parameter='target', value=target,
+                    valid_values=valid_targets)
 
-        data = {
-            'Boot': {
-                'BootSourceOverrideTarget':
-                    sys_maps.BOOT_SOURCE_TARGET_MAP_REV[target],
-                'BootSourceOverrideEnabled':
-                    sys_maps.BOOT_SOURCE_ENABLED_MAP_REV[enabled]
-            }
-        }
+            fishy_target = sys_maps.BOOT_SOURCE_TARGET_MAP_REV[target]
+            # NOTE(janders) on SuperMicro X11 and X12 machines, virtual media
+            # is presented as an "USB CD" drive as opposed to a CD drive. Both
+            # are present in the list of boot devices, however only selecting
+            # UsbCd as the boot source results in a successful boot from
+            # vMedia. If "CD" is selected, boot fails even if vMedia is
+            # inserted. This code detects a case where a SuperMicro machine is
+            # about to attempt boot from CD and overrides the boot device to
+            # UsbCd instead which makes boot from vMedia work as expected.
+            if (self.manufacturer and self.manufacturer.lower() == 'supermicro'
+                    and fishy_target == sys_maps.BOOT_SOURCE_TARGET_MAP_REV[
+                        sys_cons.BOOT_SOURCE_TARGET_CD]
+                    and sys_maps.BOOT_SOURCE_TARGET_MAP_REV[
+                        sys_cons.BOOT_SOURCE_TARGET_USB_CD]
+                    in self.boot.allowed_values):
+                fishy_target = sys_maps.BOOT_SOURCE_TARGET_MAP_REV[
+                    sys_cons.BOOT_SOURCE_TARGET_USB_CD]
+                LOG.debug('Boot from vMedia was requested on a SuperMicro'
+                          'machine. Overriding boot device from %s to %s.',
+                          sys_cons.BOOT_SOURCE_TARGET_CD,
+                          fishy_target)
+
+            data['Boot']['BootSourceOverrideTarget'] = fishy_target
+
+        if enabled is not None:
+            if enabled not in sys_maps.BOOT_SOURCE_ENABLED_MAP_REV:
+                raise exceptions.InvalidParameterValueError(
+                    parameter='enabled', value=enabled,
+                    valid_values=list(sys_maps.BOOT_SOURCE_ENABLED_MAP_REV))
+
+            fishy_freq = sys_maps.BOOT_SOURCE_ENABLED_MAP_REV[enabled]
+
+            data['Boot']['BootSourceOverrideEnabled'] = fishy_freq
 
         if mode is not None:
             if mode not in sys_maps.BOOT_SOURCE_MODE_MAP_REV:
@@ -249,12 +280,48 @@ class System(base.ResourceBase):
                     parameter='mode', value=mode,
                     valid_values=list(sys_maps.BOOT_SOURCE_MODE_MAP_REV))
 
-            data['Boot']['BootSourceOverrideMode'] = (
-                sys_maps.BOOT_SOURCE_MODE_MAP_REV[mode])
+            fishy_mode = sys_maps.BOOT_SOURCE_MODE_MAP_REV[mode]
+
+            data['Boot']['BootSourceOverrideMode'] = fishy_mode
+
+        etag = self._get_etag()
+        path = self.path
+
+        # NOTE(janders): checking if @Redfish.Settings are available; if so
+        # using the URI from the SettingsObject. Also ensuring the ETag URI
+        # matches in this case (the one above would not be valid for new URI).
+        if self._settings and self._settings.resource_uri:
+            path = self._settings.resource_uri
+            resp = self._conn.get(path)
+            etag = resp.headers.get('ETag')
 
         # TODO(lucasagomes): Check the return code and response body ?
         #                    Probably we should call refresh() as well.
-        self._conn.patch(self.path, data=data)
+        self._conn.patch(path, data=data, etag=etag)
+
+    # TODO(etingof): we should remove this method, eventually
+    def set_system_boot_source(
+            self, target, enabled=sys_cons.BOOT_SOURCE_ENABLED_ONCE,
+            mode=None):
+        """Set boot source and/or boot frequency and/or boot mode.
+
+        Set the boot source and/or boot frequency and/or boot mode to use
+        on next reboot of the System.
+
+        This method is obsoleted by `set_system_boot_options`.
+
+        :param target: The target boot source.
+        :param enabled: The frequency, whether to set it for the next
+            reboot only (BOOT_SOURCE_ENABLED_ONCE) or persistent to all
+            future reboots (BOOT_SOURCE_ENABLED_CONTINUOUS) or disabled
+            (BOOT_SOURCE_ENABLED_DISABLED).
+            Default is `BOOT_SOURCE_ENABLED_ONCE`.
+        :param mode: The boot mode (UEFI: BOOT_SOURCE_MODE_UEFI or
+            BIOS: BOOT_SOURCE_MODE_BIOS), optional.
+        :raises: InvalidParameterValueError, if any information passed is
+            invalid.
+        """
+        self.set_system_boot_options(target, enabled, mode)
 
     def set_indicator_led(self, state):
         """Set IndicatorLED to the given state.
@@ -407,6 +474,21 @@ class System(base.ResourceBase):
                                 registries=self.registries)
                 for path in paths]
 
+    @property
+    @utils.cache_it
+    def secure_boot(self):
+        """Property to reference `SecureBoot` instance
+
+        It is set once when the first time it is queried. On refresh,
+        this property is marked as stale (greedy-refresh not done).
+        Here the actual refresh of the sub-resource happens, if stale.
+        """
+        return secure_boot.SecureBoot(
+            self._conn,
+            utils.get_sub_resource_path_by(self, 'SecureBoot'),
+            redfish_version=self.redfish_version,
+            registries=self.registries)
+
 
 class SystemCollection(base.ResourceCollectionBase):
 
