@@ -27,6 +27,7 @@ from sushy.resources.oem import fake
 from sushy.resources.system import bios
 from sushy.resources.system import mappings as sys_map
 from sushy.resources.system import processor
+from sushy.resources.system import secure_boot
 from sushy.resources.system import simple_storage
 from sushy.resources.system import system
 from sushy.tests.unit import base
@@ -37,6 +38,7 @@ class SystemTestCase(base.TestCase):
     def setUp(self):
         super(SystemTestCase, self).setUp()
         self.conn = mock.Mock()
+        self.conn.get.return_value.headers = {'Allow': 'GET,HEAD'}
         with open('sushy/tests/unit/json_samples/system.json') as f:
             self.json_doc = json.load(f)
 
@@ -82,6 +84,42 @@ class SystemTestCase(base.TestCase):
         for oem_vendor in self.sys_inst.oem_vendors:
             self.assertIn(oem_vendor, ('Contoso', 'Chipwise'))
 
+    def test__parse_attributes_return(self):
+        attributes = self.sys_inst._parse_attributes(self.json_doc)
+
+        # Test that various types are returned correctly
+        self.assertEqual('Chicago-45Z-2381', attributes.get('asset_tag'))
+        self.assertEqual(sushy.INDICATOR_LED_OFF,
+                         attributes.get('indicator_led'))
+        self.assertEqual({'health': res_cons.HEALTH_OK,
+                          'health_rollup': res_cons.HEALTH_OK,
+                          'state': res_cons.STATE_ENABLED},
+                         attributes.get('status'))
+        self.assertEqual({'maintenance_window_duration_in_seconds': 1,
+                          'maintenance_window_start_time':
+                          parser.parse('2016-03-07T14:44:30-05:05')},
+                         attributes.get('maintenance_window'))
+        self.assertEqual({'reset': {'allowed_values':
+                         ['On', 'ForceOff', 'GracefulShutdown',
+                             'GracefulRestart', 'ForceRestart', 'Nmi',
+                             'ForceOn', 'PushPowerButton'],
+                         'operation_apply_time_support':
+                         {'_maintenance_window_resource':
+                             {'resource_uri':
+                                 '/redfish/v1/Systems/437XR1138R2'},
+                          'maintenance_window_duration_in_seconds': 600,
+                          'maintenance_window_start_time':
+                              parser.parse('2017-05-03T23:12:37-05:00'),
+                          'supported_values':
+                              ['Immediate', 'AtMaintenanceWindowStart'],
+                          'mapped_supported_values':
+                              [res_cons.APPLY_TIME_IMMEDIATE,
+                               res_cons.APPLY_TIME_MAINT_START]},
+                         'target_uri':
+                         '/redfish/v1/Systems/437XR1138R2/Actions/'
+                         'ComputerSystem.Reset'}},
+                         attributes.get('_actions'))
+
     def test__parse_attributes_missing_actions(self):
         self.sys_inst.json.pop('Actions')
         self.assertRaisesRegex(
@@ -173,6 +211,9 @@ class SystemTestCase(base.TestCase):
         self.assertIsNotNone(support)
         self.assertEqual(['Immediate', 'AtMaintenanceWindowStart'],
                          support.supported_values)
+        self.assertEqual([res_cons.APPLY_TIME_IMMEDIATE,
+                          res_cons.APPLY_TIME_MAINT_START],
+                         support.mapped_supported_values)
         self.assertEqual(parser.parse('2017-05-03T23:12:37-05:00'),
                          support.maintenance_window_start_time)
         self.assertEqual(600, support.maintenance_window_duration_in_seconds)
@@ -214,6 +255,7 @@ class SystemTestCase(base.TestCase):
                         sushy.BOOT_SOURCE_TARGET_PXE,
                         sushy.BOOT_SOURCE_TARGET_CD,
                         sushy.BOOT_SOURCE_TARGET_USB,
+                        sushy.BOOT_SOURCE_TARGET_USB_CD,
                         sushy.BOOT_SOURCE_TARGET_HDD,
                         sushy.BOOT_SOURCE_TARGET_BIOS_SETUP,
                         sushy.BOOT_SOURCE_TARGET_UTILITIES,
@@ -236,7 +278,8 @@ class SystemTestCase(base.TestCase):
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideTarget': 'Pxe',
-                           'BootSourceOverrideMode': 'UEFI'}})
+                           'BootSourceOverrideMode': 'UEFI'}},
+            etag=None)
 
     def test_set_system_boot_options_no_mode_specified(self):
         self.sys_inst.set_system_boot_options(
@@ -245,7 +288,8 @@ class SystemTestCase(base.TestCase):
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
-                           'BootSourceOverrideTarget': 'Hdd'}})
+                           'BootSourceOverrideTarget': 'Hdd'}},
+            etag=None)
 
     def test_set_system_boot_options_no_target_specified(self):
         self.sys_inst.set_system_boot_options(
@@ -254,7 +298,8 @@ class SystemTestCase(base.TestCase):
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
-                           'BootSourceOverrideMode': 'UEFI'}})
+                           'BootSourceOverrideMode': 'UEFI'}},
+            etag=None)
 
     def test_set_system_boot_options_no_freq_specified(self):
         self.sys_inst.set_system_boot_options(
@@ -263,12 +308,14 @@ class SystemTestCase(base.TestCase):
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideTarget': 'Pxe',
-                           'BootSourceOverrideMode': 'UEFI'}})
+                           'BootSourceOverrideMode': 'UEFI'}},
+            etag=None)
 
     def test_set_system_boot_options_nothing_specified(self):
         self.sys_inst.set_system_boot_options()
         self.sys_inst._conn.patch.assert_called_once_with(
-            '/redfish/v1/Systems/437XR1138R2', data={})
+            '/redfish/v1/Systems/437XR1138R2', data={},
+            etag=None)
 
     def test_set_system_boot_options_invalid_target(self):
         self.assertRaises(exceptions.InvalidParameterValueError,
@@ -285,6 +332,71 @@ class SystemTestCase(base.TestCase):
                 sushy.BOOT_SOURCE_TARGET_HDD,
                 enabled='invalid-enabled')
 
+    def test_set_system_boot_options_supermicro_usb_cd_boot(self):
+        (self.json_doc["Boot"]
+         ["BootSourceOverrideTarget@Redfish.AllowableValues"]).append("UsbCd")
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        self.sys_inst.manufacturer = "supermicro"
+        self.sys_inst.set_system_boot_options(
+            target=sushy.BOOT_SOURCE_TARGET_CD,
+            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'UsbCd'}},
+            etag=None)
+
+    def test_set_system_boot_options_supermicro_no_usb_cd_boot(self):
+
+        self.sys_inst.manufacturer = "supermicro"
+        self.sys_inst.set_system_boot_options(
+            target=sushy.BOOT_SOURCE_TARGET_CD,
+            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'Cd'}},
+            etag=None)
+
+    def test_set_system_boot_options_settings_resource_nokia(self):
+        self.conn.get.return_value.headers = {'ETag': '"3d7b838291941d"'}
+        with open('sushy/tests/unit/json_samples/settings-nokia.json') as f:
+            settings_obj = json.load(f)
+
+        self.json_doc.update(settings_obj)
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        self.sys_inst.set_system_boot_options(
+            target=sushy.BOOT_SOURCE_TARGET_CD,
+            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/Self/SD',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'Cd'}},
+            etag='"3d7b838291941d"')
+
+    def test_set_system_boot_options_settings_resource(self):
+        self.conn.get.return_value.headers = {'ETag': '"3d7b838291941d"'}
+        with open('sushy/tests/unit/json_samples/settings.json') as f:
+            settings_obj = json.load(f)
+
+        self.json_doc.update(settings_obj)
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        self.sys_inst.set_system_boot_options(
+            target=sushy.BOOT_SOURCE_TARGET_CD,
+            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2/BIOS/Settings',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'Cd'}},
+            etag='"3d7b838291941d"')
+
     def test_set_system_boot_source(self):
         self.sys_inst.set_system_boot_source(
             sushy.BOOT_SOURCE_TARGET_PXE,
@@ -294,7 +406,21 @@ class SystemTestCase(base.TestCase):
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideTarget': 'Pxe',
-                           'BootSourceOverrideMode': 'UEFI'}})
+                           'BootSourceOverrideMode': 'UEFI'}},
+            etag=None)
+
+    def test_set_system_boot_source_with_etag(self):
+        self.conn.get.return_value.headers = {'ETag': '"3d7b838291941d"'}
+        self.sys_inst.set_system_boot_source(
+            sushy.BOOT_SOURCE_TARGET_PXE,
+            enabled=sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
+            mode=sushy.BOOT_SOURCE_MODE_UEFI)
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
+                           'BootSourceOverrideTarget': 'Pxe',
+                           'BootSourceOverrideMode': 'UEFI'}},
+            etag='"3d7b838291941d"')
 
     def test_set_system_boot_source_no_mode_specified(self):
         self.sys_inst.set_system_boot_source(
@@ -303,7 +429,8 @@ class SystemTestCase(base.TestCase):
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
-                           'BootSourceOverrideTarget': 'Hdd'}})
+                           'BootSourceOverrideTarget': 'Hdd'}},
+            etag=None)
 
     def test_set_system_boot_source_invalid_target(self):
         self.assertRaises(exceptions.InvalidParameterValueError,
@@ -487,6 +614,15 @@ class SystemTestCase(base.TestCase):
         self.assertEqual('BIOS Configuration Current Settings',
                          self.sys_inst.bios.name)
 
+    def test_secure_boot(self):
+        self.conn.get.return_value.json.reset_mock()
+        with open('sushy/tests/unit/json_samples/secure_boot.json') as f:
+            self.conn.get.return_value.json.side_effect = [json.load(f)]
+
+        self.assertIsInstance(self.sys_inst.secure_boot,
+                              secure_boot.SecureBoot)
+        self.assertEqual('UEFI Secure Boot', self.sys_inst.secure_boot.name)
+
     def test_simple_storage_for_missing_attr(self):
         self.sys_inst.json.pop('SimpleStorage')
         with self.assertRaisesRegex(
